/* This file is part of the OWL API.
 * The contents of this file are subject to the LGPL License, Version 3.0.
 * Copyright 2014, The University of Manchester
 * 
 * This program is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License along with this program.  If not, see http://www.gnu.org/licenses/.
 *
 * Alternatively, the contents of this file may be used under the terms of the Apache License, Version 2.0 in which case, the provisions of the Apache License Version 2.0 are applicable instead of those above.
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with the License. You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the specific language governing permissions and limitations under the License. */
package org.semanticweb.owlapi.profiles;

import java.util.Arrays;
import java.util.LinkedHashSet;
import java.util.Set;

import org.semanticweb.owlapi.model.HasIRI;
import org.semanticweb.owlapi.model.OWLOntology;

/**
 * @author Matthew Horridge, The University of Manchester, Information Management Group
 */
public interface OWLProfile extends HasIRI {

    /**
     * Gets the name of the profile.
     *
     * @return A string that represents the name of the profile
     */
    String getName();

    /**
     * Checks an ontology and its import closure to see if it is within this
     * profile.
     *
     * @param ontology The ontology to be checked.
     * @return An {@code OWLProfileReport} that describes whether or not the ontology is within this
     * profile.
     */
    OWLProfileReport checkOntology(OWLOntology ontology);

    /**
     * Checks an ontology and its import closure to see if it is within the specified profiles.
     *
     * @param ontology The ontology to be checked.
     * @param profiles the profiles to check
     * @return An {@code OWLProfileReport} that describes whether or not the ontology is within the
     *         specified profiles.
     */
    default OWLProfileReport checkOntologyClosureInProfiles(OWLOntology ontology,
        Profiles... profiles) {
        Set<OWLProfileViolation> violations = new LinkedHashSet<>();
        OWLOntologyProfileWalker walker = new OWLOntologyProfileWalker(ontology.importsClosure());
        walker.walkStructure(new ProfileVisitor(walker, violations, Arrays.asList(profiles)));
        return new OWLProfileReport(this, violations);
    }
}
