/* This file is part of the OWL API.
 * The contents of this file are subject to the LGPL License, Version 3.0.
 * Copyright 2014, The University of Manchester
 * 
 * This program is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License along with this program.  If not, see http://www.gnu.org/licenses/.
 *
 * Alternatively, the contents of this file may be used under the terms of the Apache License, Version 2.0 in which case, the provisions of the Apache License Version 2.0 are applicable instead of those above.
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with the License. You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the specific language governing permissions and limitations under the License. */
package uk.ac.manchester.owl.owlapi.tutorial;

import java.util.HashSet;
import java.util.Set;

import org.semanticweb.owlapi.model.OWLAxiomVisitor;
import org.semanticweb.owlapi.model.OWLClass;
import org.semanticweb.owlapi.model.OWLSubClassOfAxiom;

/**
 * A visitor that simply collects any subclass axioms that have the given class as the subclass.
 *
 * @author Sean Bechhofer, The University Of Manchester, Information Management Group
 * @since 2.0.0
 */
public class SubClassCollector implements OWLAxiomVisitor {

    /* Collected axioms */
    private final Set<OWLSubClassOfAxiom> axioms;
    /* Class to look for */
    private final OWLClass clazz;

    /**
     * @param clazz class to search
     */
    public SubClassCollector(OWLClass clazz) {
        axioms = new HashSet<>();
        this.clazz = clazz;
    }

    @Override
    public void visit(OWLSubClassOfAxiom axiom) {
        if (axiom.getSubClass().equals(clazz)) {
            axioms.add(axiom);
        }
    }

    /**
     * @return axioms
     */
    public Set<OWLSubClassOfAxiom> getAxioms() {
        return axioms;
    }
}
